package solutions.bitbadger.documents

import java.sql.Connection
import java.sql.ResultSet

// ~~~ CUSTOM QUERIES ~~~

/**
 * Execute a query that returns a list of results
 *
 * @param query The query to retrieve the results
 * @param parameters Parameters to use for the query
 * @param mapFunc The mapping function between the document and the domain item
 * @return A list of results for the given query
 */
inline fun <reified TDoc> Connection.customList(
    query: String, parameters: Collection<Parameter<*>> = listOf(), mapFunc: (ResultSet) -> TDoc
) = Custom.list(query, parameters, this, mapFunc)

/**
 * Execute a query that returns one or no results
 *
 * @param query The query to retrieve the results
 * @param parameters Parameters to use for the query
 * @param mapFunc The mapping function between the document and the domain item
 * @return The document if one matches the query, `null` otherwise
 */
inline fun <reified TDoc> Connection.customSingle(
    query: String, parameters: Collection<Parameter<*>> = listOf(), mapFunc: (ResultSet) -> TDoc
) = Custom.single(query, parameters, this, mapFunc)

/**
 * Execute a query that returns no results
 *
 * @param query The query to retrieve the results
 * @param parameters Parameters to use for the query
 */
fun Connection.customNonQuery(query: String, parameters: Collection<Parameter<*>> = listOf()) =
    Custom.nonQuery(query, parameters, this)

/**
 * Execute a query that returns a scalar result
 *
 * @param query The query to retrieve the result
 * @param parameters Parameters to use for the query
 * @param mapFunc The mapping function between the document and the domain item
 * @return The scalar value from the query
 */
inline fun <reified T> Connection.customScalar(
    query: String,
    parameters: Collection<Parameter<*>> = listOf(),
    mapFunc: (ResultSet) -> T & Any
) = Custom.scalar(query, parameters, this, mapFunc)

// ~~~ DEFINITION QUERIES ~~~

/**
 * Create a document table if necessary
 *
 * @param tableName The table whose existence should be ensured (may include schema)
 */
fun Connection.ensureTable(tableName: String) =
    Definition.ensureTable(tableName, this)

/**
 * Create an index on field(s) within documents in the specified table if necessary
 *
 * @param tableName The table to be indexed (may include schema)
 * @param indexName The name of the index to create
 * @param fields One or more fields to be indexed<
 */
fun Connection.ensureFieldIndex(tableName: String, indexName: String, fields: Collection<String>) =
    Definition.ensureFieldIndex(tableName, indexName, fields, this)

// ~~~ DOCUMENT MANIPULATION QUERIES ~~~

/**
 * Insert a new document
 *
 * @param tableName The table into which the document should be inserted (may include schema)
 * @param document The document to be inserted
 */
inline fun <reified TDoc> Connection.insert(tableName: String, document: TDoc) =
    Document.insert(tableName, document, this)

// ~~~ DOCUMENT COUNT QUERIES ~~~

/**
 * Count all documents in the table
 *
 * @param tableName The name of the table in which documents should be counted
 * @return A count of the documents in the table
 */
fun Connection.countAll(tableName: String) =
    Count.all(tableName, this)


// ~~~ DOCUMENT RETRIEVAL QUERIES ~~~

/**
 * Retrieve all documents in the given table
 *
 * @param tableName The table from which documents should be retrieved
 * @return A list of documents from the given table
 */
inline fun <reified TDoc> Connection.findAll(tableName: String) =
    Find.all<TDoc>(tableName, this)

/**
 * Retrieve all documents in the given table
 *
 * @param tableName The table from which documents should be retrieved
 * @return A list of documents from the given table
 */
inline fun <reified TDoc> Connection.findAll(tableName: String, orderBy: Collection<Field<*>>) =
    Find.all<TDoc>(tableName, orderBy, this)
